<?php

/**
 * Intellectual Property rights, and copyright, reserved by Todd Lahman, LLC as allowed by law include,
 * but are not limited to, the working concept, function, and behavior of this software,
 * the logical code structure and expression as written.
 *
 * @package     WooCommerce API Manager plugin and theme library
 * @author      Todd Lahman LLC https://www.toddlahman.com/
 * @copyright   Copyright (c) Todd Lahman LLC (support@toddlahman.com)
 * @since       1.0
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}



if ( ! class_exists( 'venus_AM_License_Menu' ) ) {
	class venus_AM_License_Menu {

		/**
		 * Class args.
		 *
		 * @var string
		 */
		public $file             = '';
		public $software_title   = '';
		public $software_version = '';
		public $plugin_or_theme  = '';
		public $api_url          = '';
		public $data_prefix      = '';
		public $slug             = '';
		public $plugin_name      = '';
		public $text_domain      = '';
		public $extra            = '';

		/**
		 * Class properties.
		 *
		 * @var string
		 */
		public $ame_software_product_id;
		public $ame_data_key;
		public $ame_api_key;
		public $ame_activation_email;
		public $ame_product_id_key;
		public $ame_instance_key;
		public $ame_deactivate_checkbox_key;
		public $ame_activated_key;
		public $ame_activation_tab_key;
		public $ame_settings_menu_title;
		public $ame_settings_title;
		public $ame_menu_tab_activation_title;
		public $ame_menu_tab_deactivation_title;
		public $ame_options;
		public $ame_plugin_name;
		public $ame_product_id;
		public $ame_renew_license_url;
		public $ame_instance_id;
		public $ame_domain;
		public $ame_software_version;

		/**
		 * @var null
		 */
		protected static $_instance = null;

		/**
		 * @param string $file             Must be $this->file from the root plugin file, or theme functions file.
		 * @param string $software_title   Must be exactly the same as the Software Title in the product.
		 * @param string $software_version This products current software version.
		 * @param string $plugin_or_theme  'plugin' or 'theme'
		 * @param string $api_url          The URL to the site that is running the API Manager. Example: https://www.toddlahman.com/ Must have a trailing slash.
		 * @param string $text_domain      The text domain for translation. Hardcoding this string is preferred rather than using this argument.
		 * @param string $extra            Extra data. Whatever you want.
		 *
		 * @return \AM_License_Menu|null
		 */
		public static function instance( $file, $software_title, $software_version, $plugin_or_theme, $api_url, $text_domain = '', $extra = '' ) {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self( $file, $software_title, $software_version, $plugin_or_theme, $api_url, $text_domain, $extra );
			}

			return self::$_instance;
		}

		public function __construct( $file, $software_title, $software_version, $plugin_or_theme, $api_url, $text_domain, $extra ) {
			$this->file            = $file;
			$this->software_title  = $software_title;
			$this->version         = $software_version;
			$this->plugin_or_theme = $plugin_or_theme;
			$this->api_url         = $api_url;
			$this->text_domain     = $text_domain;
			$this->extra           = $extra;
			$this->data_prefix     = str_ireplace( array( ' ', '_', '&', '?' ), '_', strtolower( $this->software_title ) );

			if ( is_admin() ) {
				if ( ! empty( $this->plugin_or_theme ) && $this->plugin_or_theme == 'theme' ) {
					add_action( 'admin_init', array( $this, 'activation' ) );
				}

				if ( ! empty( $this->plugin_or_theme ) && $this->plugin_or_theme == 'plugin' ) {
					register_activation_hook( $this->file, array( $this, 'activation' ) );
				}

				add_action( 'admin_menu', array( $this, 'register_menu' ) );
				add_action( 'admin_init', array( $this, 'load_settings' ) );

				// Check for external connection blocking
				add_action( 'admin_notices', array( $this, 'check_external_blocking' ) );

				/**
				 * Software Product ID is the product title string
				 * This value must be unique, and it must match the API tab for the product in WooCommerce
				 */
				$this->ame_software_product_id = $this->software_title;

				/**
				 * Set all data defaults here
				 */
				$this->ame_data_key                = $this->data_prefix . '_data';
				$this->ame_api_key                 = 'api_key';
				$this->ame_activation_email        = 'activation_email';
				$this->ame_product_id_key          = $this->data_prefix . '_product_id';
				$this->ame_instance_key            = $this->data_prefix . '_instance';
				$this->ame_deactivate_checkbox_key = $this->data_prefix . '_deactivate_checkbox';
				$this->ame_activated_key           = $this->data_prefix . '_activated';

				/**
				 * Set all admin menu data
				 */
				$this->ame_deactivate_checkbox         = $this->data_prefix . '_deactivate_checkbox';
				$this->ame_activation_tab_key          = $this->data_prefix . '_dashboard';
				$this->ame_deactivation_tab_key        = $this->data_prefix . '_deactivation';
				$this->ame_settings_menu_title         = $this->software_title;
				$this->ame_settings_title              = $this->software_title . __( 'API Key', $this->text_domain );
				$this->ame_menu_tab_activation_title   = __( 'API Key', $this->text_domain );
				$this->ame_menu_tab_deactivation_title = __( 'API Key Deactivation', $this->text_domain );

				/**
				 * Set all software update data here
				 */
				$this->ame_options           = get_option( $this->ame_data_key );
				$this->ame_plugin_name       = $this->plugin_or_theme == 'plugin' ? untrailingslashit( plugin_basename( $this->file ) ) : get_stylesheet(); // same as plugin slug. if a theme use a theme name like 'twentyeleven'
				$this->ame_product_id        = get_option( $this->ame_product_id_key ); // Software Title
				$this->ame_renew_license_url = $this->api_url . 'my-account'; // URL to renew an API Key. Trailing slash in the upgrade_url is required.
				$this->ame_instance_id       = get_option( $this->ame_instance_key ); // Instance ID (unique to each blog activation)
				/**
				 * Some web hosts have security policies that block the : (colon) and // (slashes) in http://,
				 * so only the host portion of the URL can be sent. For example the host portion might be
				 * www.example.com or example.com. http://www.example.com includes the scheme http,
				 * and the host www.example.com.
				 * Sending only the host also eliminates issues when a client site changes from http to https,
				 * but their activation still uses the original scheme.
				 * To send only the host, use a line like the one below:
				 *
				 * $this->ame_domain = str_ireplace( array( 'http://', 'https://' ), '', home_url() ); // blog domain name
				 */
				$this->ame_domain           = str_ireplace( array( 'http://', 'https://' ), '', home_url() ); // blog domain name
				$this->ame_software_version = $this->version; // The software version
				$options                    = get_option( $this->ame_data_key );

				/**
				 * Check for software updates
				 */
				if ( ! empty( $options ) && $options !== false ) {
					$this->check_for_update();
				}

				if ( ! empty( $this->ame_activated_key ) && get_option( $this->ame_activated_key ) != 'Activated' ) {
					add_action( 'admin_notices', array( $this, 'inactive_notice' ) );
				}
			}

			/**
			 * Deletes all data if plugin deactivated
			 */
			if ( $this->plugin_or_theme == 'plugin' ) {
				register_deactivation_hook( $this->file, array( $this, 'uninstall' ) );
			}

			if ( $this->plugin_or_theme == 'theme' ) {
				add_action( 'switch_theme', array( $this, 'uninstall' ) );
			}
		}

		/**
		 * Register submenu specific to this product.
		 */
		public function register_menu() {
			add_options_page( __( $this->ame_settings_menu_title, $this->text_domain ), __( $this->ame_settings_menu_title, $this->text_domain ), 'manage_options', $this->ame_activation_tab_key, array(
				$this,
				'config_page'
			) );
		}

		/**
		 * Generate the default data arrays
		 */
		public function activation() {
			if ( get_option( $this->ame_data_key ) === false || get_option( $this->ame_instance_key ) === false ) {
				$global_options = array(
					$this->ame_api_key          => '',
					$this->ame_activation_email => '',
				);

				update_option( $this->ame_data_key, $global_options );

				$single_options = array(
					$this->ame_product_id_key          => $this->ame_software_product_id,
					$this->ame_instance_key            => wp_generate_password( 12, false ),
					$this->ame_deactivate_checkbox_key => 'on',
					$this->ame_activated_key           => 'Deactivated',
				);

				foreach ( $single_options as $key => $value ) {
					update_option( $key, $value );
				}
			}
		}

		/**
		 * Deletes all data if plugin deactivated
		 *
		 * @return void
		 */
		public function uninstall() {
			global $blog_id;

			$this->license_key_deactivation();

			// Remove options
			if ( is_multisite() ) {
				switch_to_blog( $blog_id );

				foreach (
					array(
						$this->ame_data_key,
						$this->ame_product_id_key,
						$this->ame_instance_key,
						$this->ame_deactivate_checkbox_key,
						$this->ame_activated_key,
					) as $option
				) {

					delete_option( $option );
				}

				restore_current_blog();
			} else {
				foreach (
					array(
						$this->ame_data_key,
						$this->ame_product_id_key,
						$this->ame_instance_key,
						$this->ame_deactivate_checkbox_key,
						$this->ame_activated_key
					) as $option
				) {

					delete_option( $option );
				}
			}
		}

		/**
		 * Deactivates the license on the API server
		 *
		 * @return void
		 */
		public function license_key_deactivation() {
			$activation_status = get_option( $this->ame_activated_key );
			$api_email         = $this->ame_options[ $this->ame_activation_email ];
			$api_key           = $this->ame_options[ $this->ame_api_key ];

			$args = array(
				'email'       => $api_email,
				'licence_key' => $api_key,
			);

			if ( $activation_status == 'Activated' && $api_key != '' && $api_email != '' ) {
				$this->deactivate( $args ); // reset API Key activation
			}
		}

		/**
		 * Displays an inactive notice when the software is inactive.
		 */
		public function inactive_notice() { ?>
			<?php if ( ! current_user_can( 'manage_options' ) ) {
				return;
			} ?>
			<?php if ( isset( $_GET[ 'page' ] ) && $this->ame_activation_tab_key == $_GET[ 'page' ] ) {
				return;
			} ?>
            <div class="notice notice-info">
                <p><?php printf( __( 'Venus Assistant API Key is not activated.  %sClick here%s to add API Key.', $this->text_domain ), '<a href="' . esc_url( admin_url( 'options-general.php?page=' . $this->ame_activation_tab_key ) ) . '">', '</a>' ); ?></p>
            </div>
			<?php
		}

		/**
		 * Check for external blocking contstant.
		 */
		public function check_external_blocking() {
			// show notice if external requests are blocked through the WP_HTTP_BLOCK_EXTERNAL constant
			if ( defined( 'WP_HTTP_BLOCK_EXTERNAL' ) && WP_HTTP_BLOCK_EXTERNAL === true ) {
				// check if our API endpoint is in the allowed hosts
				$host = parse_url( $this->api_url, PHP_URL_HOST );

				if ( ! defined( 'WP_ACCESSIBLE_HOSTS' ) || stristr( WP_ACCESSIBLE_HOSTS, $host ) === false ) {
					?>
                    <div class="notice notice-error">
                        <p><?php printf( __( '<b>Warning!</b> You\'re blocking external requests which means you won\'t be able to get %s updates. Please add %s to %s.', $this->text_domain ), $this->ame_software_product_id, '<strong>' . $host . '</strong>', '<code>WP_ACCESSIBLE_HOSTS</code>' ); ?></p>
                    </div>
					<?php
				}
			}
		} 

        /** 
    * Generate report
    */

    /**
     * helper function for number conversions
     *
     * @access public
     * @param mixed $v
     * @return void
     */
    public function num_convt( $v ) {
        $l   = substr( $v, -1 );
        $ret = substr( $v, 0, -1 );

        switch ( strtoupper( $l ) ) {
            case 'P': // fall-through
            case 'T': // fall-through
            case 'G': // fall-through
            case 'M': // fall-through
            case 'K': // fall-through
                $ret *= 1024;
                break;
            default:
                break;
        }

        return $ret;
    }

    public function report_data($warning_flag) {

        // call WP database
        global $wpdb;

        // data checks for later
    
        $mu_plugins = get_mu_plugins();
        $plugins    = get_plugins();
        $active     = get_option( 'active_plugins', array() );

        $theme_data = wp_get_theme();
        $theme      = $theme_data->Name . ' ' . $theme_data->Version;
        $style_parent_theme = wp_get_theme(get_template());
        $parent_theme = $style_parent_theme->get( 'Name' )." ".$style_parent_theme->get( 'Version' );
        //print_r($theme_data);

        // multisite details
        $nt_plugins = is_multisite() ? wp_get_active_network_plugins() : array();
        $nt_active  = is_multisite() ? get_site_option( 'active_sitewide_plugins', array() ) : array();
        $ms_sites   = is_multisite() ? get_blog_list() : null;

        // yes / no specifics
        $ismulti    = is_multisite() ? __( 'Yes', 'venus-report' ) : __( 'No', 'venus-report' );
        $safemode   = ini_get( 'safe_mode' ) ? __( 'Yes', 'venus-report' ) : __( 'No', 'venus-report' );
        $wpdebug    = defined( 'WP_DEBUG' ) ? WP_DEBUG ? __( 'Enabled', 'venus-report' ) : __( 'Disabled', 'venus-report' ) : __( 'Not Set', 'venus-report' );
        $errdisp    = ini_get( 'display_errors' ) != false ? __( 'On', 'venus-report' ) : __( 'Off', 'venus-report' );

        $jquchk     = wp_script_is( 'jquery', 'registered' ) ? $GLOBALS['wp_scripts']->registered['jquery']->ver : __( 'n/a', 'venus-report' );

        $sessenb    = isset( $_SESSION ) ? __( 'Enabled', 'venus-report' ) : __( 'Disabled', 'venus-report' );
        $usecck     = ini_get( 'session.use_cookies' ) ? __( 'On', 'venus-report' ) : __( 'Off', 'venus-report' );
        $hascurl    = function_exists( 'curl_init' ) ? __( 'Supports cURL.', 'venus-report' ) : __( 'Does not support cURL.', 'venus-report' );
        $openssl    = extension_loaded('openssl') ? __( 'OpenSSL installed.', 'venus-report' ) : __( 'OpenSSL not installed.', 'venus-report' );

        // language

        $site_lang = get_bloginfo('language');
        if (is_rtl()) $site_text_dir = 'rtl';
        	else $site_text_dir = 'ltr';

        // start generating report

        global $wpdb;
    	$table_name =  $wpdb->prefix.'options';

        $report = '<div id="venus-report">';
        $report .= '<h2>Understanding The System Status Report</h2>';
        $report .= '<p><h4>Self Help</h4> The system report contains useful technical information to predict problems. If you see an orange "Warning" notice, it can indicate a potential issue. To ensure optimal performance of the plugin, please work with your hosting company support. Update server settings to match the "Recommended Values" tab. You can take a screenshot or copy/paste the recommended and actual values and ask your hosting company to update the setting/s for you. </p><p><h4>Divi Den Support</h4> When submitting a support request, use the "Copy Report to Clipboard" button and paste it in <a target="_blank" href="https://seku.re/support">your message to support</a>.</p>';
        $report .= '<input data-clipboard-action="copy" data-clipboard-target="#venus-report-textarea" id="venus-copy-report" type="button" value="Copy Report to Clipboard" class="et-core-modal-action">';
        $report .= '<p id="venus-success-report" class="notice notice-success" style="max-width: 235px; margin-top: 10px; margin-bottom: 0;">Done: Copied to clipboard</p>';
        $report .= '<textarea readonly="readonly" id="venus-report-textarea" name="venus-report-textarea" style="width:0; height: 0; margin 0; padding: 0 !important; margin-top: -15px; position: absolute; z-index: -1; ">';
        $report .= '====== BEGIN REPORT ======'."\n";

        $report .= "\n".'--- WORDPRESS DATA ---'."\n";
        $report .= 'Multisite:'." ".$ismulti."\n";
        $report .= 'SITE_URL:'." ".site_url()."\n";
        $report .= 'HOME_URL:'." ".home_url()."\n";
        $report .= 'WP Version:'." ".get_bloginfo( 'version' )."\n";
        $report .= 'Permalink:'." ".get_option( 'permalink_structure' )."\n";
        $report .= 'Current Theme:'." ".$theme."\n";
        $report .= 'Parent Theme:'." ".$parent_theme."\n";

        $report .= "\n".'--- WORDPRESS CONFIG ---'."\n";
        $report .= 'WP_DEBUG:'." ".$wpdebug."\n";
        $report .= 'WP Memory Limit:'." ".$this->num_convt( WP_MEMORY_LIMIT )/( 1024 ).'MB'."\n";
        $report .= 'jQuery Version:'." ".$jquchk."\n";
        $report .= 'Site Language:'." ".$site_lang."\n";
        $report .= 'Site Text Direction:'." ".$site_text_dir."\n";

        if ( is_multisite() ) :
            $report .= "\n".'--- MULTISITE INFORMATION ---'."\n";
            $report .= 'Total Sites:'." ".get_blog_count()."\n";
            $report .= 'Base Site:'." ".$ms_sites[0]['domain']."\n";
            $report .= 'All Sites:'."\n";
            foreach ( $ms_sites as $site ) :
                if ( $site['path'] != '/' )
                    $report .= " ".'- '. $site['domain'].$site['path']."\n";

            endforeach;
            $report .= "\n";
        endif;

        $report .= "\n".'--- SERVER DATA ---'."\n";
        $report .= 'PHP Version:'." ".PHP_VERSION."\n";
        $report .= 'Server Software:'." ".$_SERVER['SERVER_SOFTWARE']."\n";

        $report .= "\n".'--- PHP CONFIGURATION ---'."\n";
        $report .= 'Safe Mode:'." ".$safemode."\n";
        $report .= 'memory_limit:'." ".ini_get( 'memory_limit' )."\n";
        $report .= 'upload_max_filesize:'." ".ini_get( 'upload_max_filesize' )."\n";
        $report .= 'post_max_size:'." ".ini_get( 'post_max_size' )."\n";
        $report .= 'max_execution_time:'." ".ini_get( 'max_execution_time' )."\n";
        $report .= 'max_input_vars:'." ".ini_get( 'max_input_vars' )."\n";
        $report .= 'max_input_time:'." ".ini_get( 'max_input_time' )."\n";
        $report .= 'Display Errors:'." ".$errdisp."\n";
        $report .= 'Cookie Path:'." ".esc_html( ini_get( 'session.cookie_path' ) )."\n";
        $report .= 'Save Path:'." ".esc_html( ini_get( 'session.save_path' ) )."\n";
        $report .= 'Use Cookies:'." ".$usecck."\n";
        $report .= 'cURL:'." ".$hascurl."\n";
        $report .= 'OpenSSL:'." ".$openssl."\n";

        $report .= "\n".'--- PLUGIN INFORMATION ---'."\n";
        if ( $plugins && $mu_plugins ) :
            $report .= 'Total Plugins:'." ".( count( $plugins ) + count( $mu_plugins ) + count( $nt_plugins ) )."\n";
        endif;

        // output must-use plugins
        if ( $mu_plugins ) :
            $report .= 'Must-Use Plugins: ('.count( $mu_plugins ).')'. "\n";
            foreach ( $mu_plugins as $mu_path => $mu_plugin ) :
                $report .= "\t".'- '.$mu_plugin['Name'] . ' ' . $mu_plugin['Version'] ."\n";
            endforeach;
            $report .= "\n";
        endif;

        // if multisite, grab active network as well
        // if ( is_multisite() ) :
        //     // active network
        //     $report .= 'Network Active Plugins: ('.count( $nt_plugins ).')'. "\n";

        //     foreach ( $nt_plugins as $plugin_path ) :
        //         if ( array_key_exists( $plugin_base, $nt_plugins ) )
        //             continue;

        //         $plugin = get_plugin_data( $plugin_path );

        //         $report .= "\t".'- '.$plugin['Name'] . ' ' . $plugin['Version'] ."\n";
        //     endforeach;
        //     $report .= "\n";

        // endif;

        // output active plugins
        if ( $plugins ) :
            $report .= 'Active Plugins: ('.count( $active ).')'. "\n";
            foreach ( $plugins as $plugin_path => $plugin ) :
                if ( ! in_array( $plugin_path, $active ) )
                    continue;
                $report .= "\t".'- '.$plugin['Name'] . ' ' . $plugin['Version'] ."\n";
            endforeach;
            $report .= "\n";
        endif;

        // output inactive plugins
        if ( $plugins ) :
            $report .= 'Inactive Plugins: ('.( count( $plugins ) - count( $active ) ).')'. "\n";
            foreach ( $plugins as $plugin_path => $plugin ) :
                if ( in_array( $plugin_path, $active ) )
                    continue;
                $report .= "\t".'- '.$plugin['Name'] . ' ' . $plugin['Version'] ."\n";
            endforeach;
            $report .= "\n";
        endif;

        // end it all
        $report .= "\n".'====== END REPORT ======';
        $report .= '</textarea></div>';

        $venus_warning_status = '<td class="venus_warning"><span>Warning</span></td>';
        $venus_ok_status = '<td class="venus_ok"><span>OK</span></td>';

        $report_table = '';

        $report_table .= '<table class="venus-report-table"><tr><th colspan="4">Server Environment</th><tr class="venus-header-row"><td>Config Option</td><td>Recommended Value</td><td>Actual Value</td><td>Status</td></tr>';

        $report_table .= '<tr><td>PHP Version</td><td>5.2+ (7.0+ is better)</td><td>'.PHP_VERSION.'</td>';
        if((int)str_replace(".", "",PHP_VERSION) >= 52) $venus_php_status = $venus_ok_status;
            else $venus_php_status = $venus_warning_status;
            $report_table .= $venus_php_status.'</tr>';

        $report_table .= '<tr><td>Server Software</td><td>-</td><td>'.$_SERVER['SERVER_SOFTWARE'].'</td>'.$venus_ok_status.'</tr>';

        $report_table .= '<tr><td>Safe Mode</td><td>No</td><td>'.$safemode.'</td>';
        if($safemode == 'No') $venus_safe_mode_status = $venus_ok_status;
            else $venus_safe_mode_status = $venus_warning_status;
            $report_table .= $venus_safe_mode_status.'</tr>';

        $report_table .= '<tr><td>memory_limit</td><td>256+ MB</td><td>'.ini_get( 'memory_limit' ).'B</td>';
        if((int)str_replace("M", "",ini_get( 'memory_limit' )) >= 256) $venus_memory_limit_status = $venus_ok_status;
            else $venus_memory_limit_status = $venus_warning_status;
            $report_table .= $venus_memory_limit_status.'</tr>';

        $report_table .= '<tr><td>post_max_size</td><td>64+ MB</td><td>'.ini_get( 'post_max_size' ).'B</td>';
        if((int)str_replace("M", "",ini_get( 'post_max_size' )) >= 64) $venus_post_max_size_status = $venus_ok_status;
            else $venus_post_max_size_status = $venus_warning_status;
            $report_table .= $venus_post_max_size_status.'</tr>';

        $report_table .= '<tr><td>max_execution_time</td><td>180+</td><td>'.ini_get( 'max_execution_time' ).'</td>';
        if((int)ini_get( 'max_execution_time') >= 180) $venus_max_execution_time_status = $venus_ok_status;
            else $venus_max_execution_time_status = $venus_warning_status;
            $report_table .= $venus_max_execution_time_status.'</tr>';

         $report_table .= '<tr><td>upload_max_filesize</td><td>64+ MB</td><td>'.ini_get( 'upload_max_filesize' ).'B</td>';
        if((int)str_replace("M", "",ini_get( 'upload_max_filesize' )) >= 64) $venus_upload_max_status = $venus_ok_status;
            else $venus_upload_max_status = $venus_warning_status;
            $report_table .= $venus_upload_max_status.'</tr>';

        $report_table .= '<tr><td>max_input_time</td><td>300+</td><td>'.ini_get( 'max_input_time' ).'</td>';
        if((int)ini_get( 'max_input_time') >= 300) $venus_max_input_time_status = $venus_ok_status;
            else $venus_max_input_time_status = $venus_warning_status;
            $report_table .= $venus_max_input_time_status.'</tr>';

        $report_table .= '<tr><td>max_input_vars</td><td>3000+</td><td>'.ini_get( 'max_input_vars' ).'</td>';
        if((int)ini_get( 'max_input_vars') >= 3000) $venus_max_input_vars_status = $venus_ok_status;
            else $venus_max_input_vars_status = $venus_warning_status;
            $report_table .= $venus_max_input_vars_status.'</tr>';

        $report_table .= '<tr><td>Display Errors</td><td>Off</td><td>'.$errdisp.'</td>';
        if($errdisp == 'Off') $venus_display_errors_status = $venus_ok_status;
            else $venus_display_errors_status = $venus_warning_status;
            $report_table .= $venus_display_errors_status.'</tr>';

        $report_table .= '<tr><td>Cookie Path</td><td>-</td>'.'<td>'.esc_html( ini_get( 'session.cookie_path' ) ).'</td>'.$venus_ok_status.'</tr>';

        $report_table .= '<tr><td>Save Path</td><td>-</td>'.'<td>'.esc_html( ini_get( 'session.save_path' ) ).'</td>'.$venus_ok_status.'</tr>';

        $report_table .= '<tr><td>Use Cookies</td><td>On</td><td>'.$usecck.'</td>';
        if($usecck == 'On') $venus_use_cookies_status = $venus_ok_status;
            else $venus_use_cookies_status = $venus_warning_status;
            $report_table .= $venus_use_cookies_status.'</tr>';

        $report_table .= '<tr><td>cURL</td><td>Supports cURL.</td><td>'.$hascurl.'</td>';
        if($hascurl == 'Supports cURL.') $venus_curl_status = $venus_ok_status;
            else $venus_curl_status = $venus_warning_status;
            $report_table .= $venus_curl_status.'</tr>';

        $report_table .= '<th colspan="4">WordPress Environment</th></tr><tr class="venus-header-row"><td>Config Option</td><td>Recommended Value</td><td>Actual Value</td><td>Status</td></tr>';
        $report_table .= '<tr><td>Multisite</td><td>-</td>';
        $report_table .= '<td>'.$ismulti.'</td>'.$venus_ok_status.'</tr>';
        $report_table .= '<td>Site url</td><td>-</td>'.'<td>'.site_url().'</td>'.$venus_ok_status.'</tr>';
        $report_table .= '<td>Home url</td><td>-</td>'.'<td>'.home_url().'</td>'.$venus_ok_status.'</tr>';
        $report_table .= '<td>WP Version</td><td>4.2+</td><td>'.get_bloginfo( 'version' ).'</td>';
        if ( (int)str_replace(".", "", get_bloginfo( 'version' )) < 42) {$wp_version_status = $venus_warning_status;}
            else {$wp_version_status = $venus_ok_status;}
        $report_table .= $wp_version_status.'</tr>';
        
        $report_table .= '<td>Permalink</td><td>-</td>'.'<td>'.get_option( 'permalink_structure' ).'</td>'.$venus_ok_status.'</tr><tr>';
        $report_table .= '<td>Current Theme</td><td>-</td>'.'<td>'.$theme.'</td>'.$venus_ok_status.'</tr><tr>';
       
        $report_table .= '<td>Parent Theme</td><td>Divi 3.0.0+</td>'.'<td>'.$parent_theme.'</td>';
         if ( $style_parent_theme->get( 'Name' ) != 'Divi' || (int)str_replace(".", "", $style_parent_theme->get( 'Version' )) < 300) {$wp_parent_theme_status = $venus_warning_status;}
            else {$wp_parent_theme_status = $venus_ok_status;}
        $report_table .= $wp_parent_theme_status.'</tr>';
        
        $report_table .= '<tr><td>WP debug</td><td>Disabled</td><td>'.$wpdebug.'</td>';
        if($wpdebug == 'Disabled') $venus_wpdebug_status = $venus_ok_status;
            else $venus_wpdebug_status = $venus_warning_status;
            $report_table .= $venus_wpdebug_status.'</tr>';

        $report_table .= '<tr><td>WP Memory Limit</td><td>30+ MB</td><td>'.$this->num_convt( WP_MEMORY_LIMIT )/( 1024 ).' MB</td>';
             if($this->num_convt( WP_MEMORY_LIMIT )/( 1024 ) > 30) $venus_wp_memory_status = $venus_ok_status;
            else $venus_wp_memory_status = $venus_warning_status;
        $report_table .= $venus_wp_memory_status.'</tr>';

        $report_table .= '<tr><td>jQuery Version</td><td>1.1.0+</td><td>'.$jquchk.'</td>';
        if((int)str_replace(".", "",$jquchk) >= 110) $venus_jquery_status = $venus_ok_status;
            else $venus_jquery_status = $venus_warning_status;
            $report_table .= $venus_jquery_status.'</tr>';

        $report_table .= '<td>Site Language</td><td>-</td>'.'<td>'.$site_lang.'</td>'.$venus_ok_status.'</tr><tr>';

        $report_table .= '<td>Site Text Direction</td><td>ltr (left-to-right)</td>'.'<td>'.$site_text_dir.'</td>';

        if($site_text_dir == 'ltr') $venus_mtd_status = $venus_ok_status;
            else $venus_mtd_status = $venus_warning_status;
            $report_table .= $venus_mtd_status.'</tr>';


            // multisite info
		if ( is_multisite() ) :
			$report_table .= '<th colspan="4">Multisite</th></tr><tr class="venus-header-row"><td>Config Option</td><td>Recommended Value</td><td>Actual Value</td><td>Status</td></tr>';
        $report_table .= '<tr><td>Total Sites</td><td>-</td>';
        $report_table .= '<td>'.get_blog_count().'</td>'.$venus_ok_status.'</tr>';
        $report_table .= '<td>Base Site</td><td>-</td>'.'<td>'.$ms_sites[0]['domain'].'</td>'.$venus_ok_status.'</tr>';
        $report_table .= '<td colspan="2">All Sites</td>'.'<td colspan="2">';
        	foreach ( $ms_sites as $site ) :
                if ( $site['path'] != '/' ) {
                    $report_table .= $site['domain'];
                	$report_table .= $site['path'];
                	$report_table .= "<br/>";
                }
          	endforeach;
        $report_table .='</td></tr>';
   
        endif;      // is_multisite()  

         // output active plugins
        if ($plugins ):

        	$report_table .= '<th colspan="4">Active plugins</th>';

        	$report_table .= '<tr><td colspan="2">';
        	$report_table .= count( $active );
        	$report_table .= ' active plugins</td><td colspan="2">';
         	foreach ( $plugins as $plugin_path => $plugin ) :
                if ( ! in_array( $plugin_path, $active ) )
                    continue;
                $report_table .= $plugin['Name'];
                $report_table .= ' ';
                $report_table .= $plugin['Version'];
                $report_table .= "<br>";
         	endforeach;
         	$report_table .= '</td></tr>';

        endif;

         // output inctive plugins
        if ($plugins ):

        	$report_table .= '<th colspan="4">Inactive Plugins</th>';

        	$report_table .= '<tr><td colspan="2">';
        	$report_table .=  count( $plugins ) - count( $active );
        	$report_table .= ' inactive plugins</td><td colspan="2">';
         	foreach ( $plugins as $plugin_path => $plugin ) :
                if ( in_array( $plugin_path, $active ) )
                    continue;
                $report_table .= $plugin['Name'];
                $report_table .= ' ';
                $report_table .= $plugin['Version'];
                $report_table .= "<br>";
         	endforeach;
         	$report_table .= '</td></tr></table>';

        endif;

        if (strpos($report_table, $venus_warning_status) !== false) {
        	$class = 'notice notice-warning is-dismissible';
			$message = 'Action Required: Please review Venus Assistant system status report. Setting update may be required for best results. <a href=?page=' . $this->ame_activation_tab_key . '&tab=venus_assistant_system_status>Go to system status tab</a>.';

	if($warning_flag == 1 && PAnD::is_admin_notice_active( 'disable-venus-status-report-notice-forever' ) )  
		printf( '<div data-dismissible="disable-venus-status-report-notice-forever" class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $message ); 
        }

        return $report.$report_table;
    }

    public function venus_getting_started() {
  
    	$venus_starting = '<div><h2>Get a brief overview and list of tutorial videos here</h2>
            <a class="et-core-modal-action" target="_blank" href="https://seku.re/venus-kb">Knowledge Base: Go to - Getting Started With Venus Pack</a>
            <h2>How to make custom CSS changes</h2>
            <p>Open the master CSS changer file in your browser window (link below). You can copy/paste all content into a text editor and save the file to your local machine, then make edits there. Or you can copy/paste matching rules directly from the page and add them to the Divi Custom CSS box, then change the colour there before saving. Whichever method you prefer. Read the labels/comments in the file and find the Venus Pack module name and rules you want to change.</p> 
            <p>Replace the relevant colour for the matching CSS rule - then copy/paste the complete CSS rule into the "Custom CSS" box on the page edit screen OR in the <a target="_blank" href="';
            $venus_starting .= get_site_url();
            $venus_starting .= '/wp-admin/admin.php?page=et_divi_options#divi_custom_css">theme settings "Custom CSS" box</a>. Save and update the page or settings.</p>
            <p>Please watch one of the many videos if you are unsure how or where to do this.</p>
            <h4>Changes not showing?</h4>
            <p>Remember to clear all types of caches if your changes do not appear. See <a target="_blank" href="https://divi-den.helpscoutdocs.com/article/39-6-kinds-of-caches-to-clear">6 kinds of caches to clear</a></p>
            <p>We suggest to deactivate caching temporarily: <a target="_blank" href="https://divi-den.helpscoutdocs.com/article/23-deactivate-caching">Why you should never build/develop websites with caching activated</a><br/><br/>
            <a class="et-core-modal-action" target="_blank"  href="https://s3-eu-west-1.amazonaws.com/ddlicensed/venus/master-css-changer-venus-ui-kit.css">Open Master CSS Changer File In Browser Window</a></div>';
            return $venus_starting;
    }

    public function venus_assistant_help_faq() {
    	$venus_help_faq = '<br/><a class="et-core-modal-action" target="_blank"  href="https://seku.re/kb">Go to knowledge base</a><br/><br/><a class="et-core-modal-action" target="_blank"  href="https://seku.re/support">Submit support request</a>';
    	return $venus_help_faq;
    }

    public function venus_assistant_load_css_func() {
    	if (!get_option( 'venus_blog_modules')) add_option( 'venus_blog_modules', 'et_pb_on_state' );

if (!get_option( 'venus_blurbs_modules')) add_option( 'venus_blurbs_modules', 'et_pb_on_state' );

if (!get_option( 'venus_cta_modules')) add_option( 'venus_cta_modules', 'et_pb_on_state' );

if (!get_option( 'venus_form_modules')) add_option( 'venus_form_modules', 'et_pb_on_state' );

if (!get_option( 'venus_faq_modules')) add_option( 'venus_faq_modules', 'et_pb_on_state' );

if (!get_option( 'venus_features_modules')) add_option( 'venus_features_modules', 'et_pb_on_state' );

if (!get_option( 'venus_header_modules')) add_option( 'venus_header_modules', 'et_pb_on_state' );

if (!get_option( 'venus_person_modules')) add_option( 'venus_person_modules', 'et_pb_on_state' );

if (!get_option( 'venus_pricig_tables_modules')) add_option( 'venus_pricig_tables_modules', 'et_pb_on_state' );

if (!get_option( 'venus_subscribe_modules')) add_option( 'venus_subscribe_modules', 'et_pb_on_state' );

    	$load_css_page_html = '<h1>Control how CSS / Scripts load</h1><h2>Reduce unused code (optional)</h2><p>Use slider buttons to selectively load CSS & Scripts for Venus UI Kit Modules. When deactivated CSS & Scripts will not load in the source code of your website.</p><h2>Important! Avoid unexpected issues</h2><p>Venus modules will "appear broken" if CSS & Scripts have been deactivated. Therefore we recommend that you deactivate modules only as a final step, before launching the website. If newly added modules "appear broken", the CSS & Scripts may not be loading. Review the settings here and reactivate modules if required.</p><div class="divi-button">';

    	$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Blog Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_blog_modules" id="venus_blog_modules" style="display: none;"><div class="et_pb_yes_no_button ';
    	$load_css_page_html .= get_option( 'venus_blog_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Blurbs Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_blurbs_modules" id="venus_blurbs_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_blurbs_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Call to Action Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_cta_modules" id="venus_cta_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_cta_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Contact Form Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_form_modules" id="venus_form_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_form_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus FAQ Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_faq_modules" id="venus_faq_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_faq_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Features Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_features_modules" id="venus_features_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_features_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Header Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_header_modules" id="venus_header_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_header_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Person Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_person_modules" id="venus_person_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_person_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Pricing Tables Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_pricig_tables_modules" id="venus_pricig_tables_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_pricig_tables_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

		$load_css_page_html .= '<div class="et-epanel-box"><div class="et-box-title"><h3>Venus Subscribe Modules</h3><p class="et-box-subtitle"></p></div><div class="et-box-content"><input type="checkbox" class="et-checkbox yes_no_button" name="venus_subscribe_modules" id="venus_subscribe_modules" style="display: none;"><div class="et_pb_yes_no_button ';
		$load_css_page_html .= get_option( 'venus_subscribe_modules' );
    	$load_css_page_html .= '"><!-- .et_pb_on_state || .et_pb_off_state -->
				<span class="et_pb_value_text et_pb_on_value">Enabled</span>
				<span class="et_pb_button_slider"></span>
				<span class="et_pb_value_text et_pb_off_value">Disabled</span>
			</div></div></div>';

    	$load_css_page_html .= '<p class="submit"><input type="submit" name="submit" id="submit-load-css" class="button button-primary" value="Save Changes"></p><p id="venus-success-css-load" class="notice notice-success" style="max-width: 235px; margin-top: 10px; margin-bottom: 0px; display: none;">Done: Settings saved</p>';
    	$load_css_page_html .= '</div>';
    	return $load_css_page_html;
    }

    public function venus_options_count() {
    	$venus_count = 0;

    	return $venus_count;
    }

		// Draw option page
	public function config_page() {
		$venus_count = 0;
		if(get_option( 'venus_header_modules') == 'et_pb_on_state') {
        	$venus_count++;
	    }
	    if(get_option( 'venus_form_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }

	    if(get_option( 'venus_pricig_tables_modules') == 'et_pb_on_state') {
	       $venus_count++;
	    }

	    if(get_option( 'venus_features_modules') == 'et_pb_on_state') {
	       $venus_count++;
	    }

	    if(get_option( 'venus_blog_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }

	    if(get_option( 'venus_person_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }

	    if(get_option( 'venus_faq_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }

	    if(get_option( 'venus_cta_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }

	    if(get_option( 'venus_subscribe_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }

	    if(get_option( 'venus_blurbs_modules') == 'et_pb_on_state') {
	        $venus_count++;
	    }


		$this->report_data($warning_flag = 1);
			$settings_tabs = array( 
				'venus_assistant_getting_started' => 'Get Started',  'venus_assistant_system_status' => 'System Status / Troubleshooting',
				$this->ame_activation_tab_key   => __( $this->ame_menu_tab_activation_title, $this->text_domain ),
				'venus_assistant_help_faq' => 'Help / Feedback',
				'venus_assistant_load_css' => 'Selectively Load CSS / Scripts ('.$venus_count.' - 10 Enabled)'
			);
			if (get_option( 'venus_assistant_activated' ) == 'Activated') {
				if(isset( $_GET[ 'tab' ] )) $current_tab = $tab = $_GET[ 'tab' ];
					else $current_tab = $tab = 'venus_assistant_getting_started';
			}
			else {
			if(isset( $_GET[ 'tab' ] )) $current_tab = $tab = $_GET[ 'tab' ];
			else $current_tab = $tab = $this->ame_activation_tab_key; }
			?>
            <div class='wrap venus-assistant <?php echo strtolower(get_option("venus_assistant_activated"))?>'>
            	<h1>Venus Assistant Dashboard</h1>
                <h2 class="nav-tab-wrapper">
					<?php
					foreach ( $settings_tabs as $tab_page => $tab_name ) {
						$active_tab = $current_tab == $tab_page ? 'nav-tab-active' : '';
						echo '<a class="nav-tab '. $tab_page . ' ' .$active_tab . '" href="?page=' . $this->ame_activation_tab_key . '&tab=' . $tab_page . '">' . $tab_name . '</a>';
					}
					?>
                </h2>
                <form action='options.php' method='post'>
                    <div class="main">
						<?php
						if ( $tab == $this->ame_activation_tab_key ) {
							settings_fields( $this->ame_data_key );
							do_settings_sections( $this->ame_activation_tab_key );
							submit_button( __( 'Save Changes', $this->text_domain ) );
							if (get_option( 'venus_assistant_activated' ) == 'Activated' ) {
								settings_fields( $this->ame_deactivate_checkbox );
								do_settings_sections( $this->ame_deactivation_tab_key );
								submit_button( __( 'Save Changes', $this->text_domain ) );
						}
						}
						else if ( $tab == 'venus_assistant_load_css' ) {
							echo $this->venus_assistant_load_css_func();
						}
						else if ( $tab == 'venus_assistant_system_status') {
							echo $this->report_data($warning_flag = 0);
						}
						else if ( $tab == 'venus_assistant_getting_started' && get_option( 'venus_assistant_activated' ) == 'Activated') {
						
						//	echo get_option( 'venus_page_transition_duration' );
							echo $this->venus_getting_started();
						}
						else if ( $tab == 'venus_assistant_help_faq' && get_option( 'venus_assistant_activated' ) == 'Activated') {
							echo $this->venus_assistant_help_faq();
						}
						?>
                    </div>
                </form>
            </div>
			<?php
		}



		// Register settings
		public function load_settings() {
			register_setting( $this->ame_data_key, $this->ame_data_key, array( $this, 'validate_options' ) );
			// API Key
			add_settings_section( $this->ame_api_key, __( 'API Key Activation', $this->text_domain ), array(
				$this,
				'wc_am_api_key_text'
			), $this->ame_activation_tab_key );

		//	register_setting( 'venus_loading', 'venus_css_load');
			// API Key
		//	add_settings_section( 'venus_loading_css', __( 'Selectively Load CSS', $this->text_domain ), array(
		//		$this,
		//		'venus_css_text'
		//	), $this->'venus_assistant_load_css' );
		//	add_settings_field( 'status', __( 'API Key Status', $this->text_domain ), array(
		//		$this,
		//		'wc_am_api_key_status'
		//	), $this->ame_activation_tab_key, $this->ame_api_key );

			add_settings_field( 'status', __( 'API Key Status', $this->text_domain ), array(
				$this,
				'wc_am_api_key_status'
			), $this->ame_activation_tab_key, $this->ame_api_key );
			add_settings_field( $this->ame_api_key, __( 'API Key', $this->text_domain ), array(
				$this,
				'wc_am_api_key_field'
			), $this->ame_activation_tab_key, $this->ame_api_key );
			add_settings_field( $this->ame_activation_email, __( 'API Email', $this->text_domain ), array(
				$this,
				'wc_am_api_email_field'
			), $this->ame_activation_tab_key, $this->ame_api_key );
			// Activation settings
			register_setting( $this->ame_deactivate_checkbox, $this->ame_deactivate_checkbox, array( $this, 'wc_am_license_key_deactivation' ) );
			add_settings_section( 'deactivate_button', __( 'API Key Deactivation', $this->text_domain ), array(
				$this,
				'wc_am_deactivate_text'
			), $this->ame_deactivation_tab_key );
			add_settings_field( 'deactivate_button', __( 'Deactivate API Key', $this->text_domain ), array(
				$this,
				'wc_am_deactivate_textarea'
			), $this->ame_deactivation_tab_key, 'deactivate_button' );
		}

		// Provides text for api key section
		public function wc_am_api_key_text() { 
		echo '<h4>Please input the API Key and API Email below. You can find this information under "My Account / API Keys" when you <a target="_blank" href="https://seku.re/api-key">login to your account</a>. If you need assistance with API Keys or activation <a target="_blank" href="https://seku.re/support">please contact support</a>.</h4>'; }

		// Returns the API Key status from the WooCommerce API Manager on the server
		public function wc_am_api_key_status() {
			$license_status       = $this->license_key_status();
			$license_status_check = ( ! empty( $license_status[ 'status_check' ] ) && $license_status[ 'status_check' ] == 'active' ) ? '<span id="venus-active">Activated <span class="dashicons dashicons-yes" style="color: #66ab03;"></span></span>' : '<span id="venus-not-active">Deactivated <span class="dashicons dashicons-no" style="color: #ca336c;"></span></span>';
			if ( ! empty( $license_status_check ) ) {
				echo $license_status_check;
			}
		}

		// Returns API Key text field
		public function wc_am_api_key_field() {
			echo "<input id='api_key' name='" . $this->ame_data_key . "[" . $this->ame_api_key . "]' size='25' type='text' value='" . $this->ame_options[ $this->ame_api_key ] . "' />";
			if ( $this->ame_options[ $this->ame_api_key ] ) {
				echo "<span class='dashicons dashicons-yes' style='color: #66ab03;'></span>";
			} else {
				echo "<span class='dashicons dashicons-no' style='color: #ca336c;'></span>";
			}
		}

		// Returns API email text field
		public function wc_am_api_email_field() {
			echo "<input id='activation_email' name='" . $this->ame_data_key . "[" . $this->ame_activation_email . "]' size='25' type='text' value='" . $this->ame_options[ $this->ame_activation_email ] . "' />";
			if ( $this->ame_options[ $this->ame_activation_email ] ) {
				echo "<span class='dashicons dashicons-yes' style='color: #66ab03;'></span>";
			} else {
				echo "<span class='dashicons dashicons-no' style='color: #ca336c;'></span>";
			}
		}

		// Sanitizes and validates all input and output for Dashboard
		public function validate_options( $input ) {
			// Load existing options, validate, and update with changes from input before returning
			$options                                = $this->ame_options;
			$options[ $this->ame_api_key ]          = trim( $input[ $this->ame_api_key ] );
			$options[ $this->ame_activation_email ] = trim( $input[ $this->ame_activation_email ] );
			$api_email                              = trim( $input[ $this->ame_activation_email ] );
			$api_key                                = trim( $input[ $this->ame_api_key ] );
			$activation_status                      = get_option( $this->ame_activated_key );
			$checkbox_status                        = get_option( $this->ame_deactivate_checkbox );
			$current_api_key                        = $this->ame_options[ $this->ame_api_key ];

			// Should match the settings_fields() value
			if ( $_REQUEST[ 'option_page' ] != $this->ame_deactivate_checkbox ) {
				if ( $activation_status == 'Deactivated' || $activation_status == '' || $api_key == '' || $api_email == '' || $checkbox_status == 'on' || $current_api_key != $api_key ) {
					/**
					 * If this is a new key, and an existing key already exists in the database,
					 * deactivate the existing key before activating the new key.
					 */
					if ( $current_api_key != $api_key ) {
						$this->replace_license_key( $current_api_key );
					}

					$args             = array(
						'email'       => $api_email,
						'licence_key' => $api_key,
					);
					$activate_results = json_decode( $this->activate( $args ), true );

					if ( $activate_results[ 'activated' ] === true && ! empty( $this->ame_activated_key ) ) {
						add_settings_error( 'activate_text', 'activate_msg', sprintf( __( '%s activated. ', $this->text_domain ), esc_attr( $this->software_title ) ), 'updated');
						update_option( $this->ame_activated_key, 'Activated' );
						update_option( $this->ame_deactivate_checkbox, 'off' );
					}

					if ( $activate_results == false && ! empty( $this->ame_options ) && ! empty( $this->ame_activated_key ) ) {
						//add_settings_error( 'api_key_check_text', 'api_key_check_error', __( 'Connection failed to the API Key server. Try again later. There may be a problem on your server preventing outgoing requests, or the store is blocking your request to activate the plugin/theme.', $this->text_domain ), 'error' );
						//$options[ $this->ame_api_key ]          = '';
						//$options[ $this->ame_activation_email ] = '';
						update_option( $this->ame_activated_key, 'Activated' );
						update_option( $this->ame_deactivate_checkbox, 'off' );
					}

					if ( isset( $activate_results[ 'code' ] ) && ! empty( $this->ame_options ) && ! empty( $this->ame_activated_key ) ) {
						switch ( $activate_results[ 'code' ] ) {
							case '100':
								$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								add_settings_error( 'api_email_text', 'api_email_error', "{$activate_results['error']}. {$additional_info}", 'error' );
								$options[ $this->ame_activation_email ] = '';
								$options[ $this->ame_api_key ]          = '';
								update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
								break;
							case '101':
								$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								add_settings_error( 'api_key_text', 'api_key_error', "{$activate_results['error']}. {$additional_info}", 'error' );
								$options[ $this->ame_api_key ]          = '';
								$options[ $this->ame_activation_email ] = '';
								update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
								break;
							case '102':
								$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								add_settings_error( 'api_key_purchase_incomplete_text', 'api_key_purchase_incomplete_error', "{$activate_results['error']}. {$additional_info}", 'error' );
								$options[ $this->ame_api_key ]          = '';
								$options[ $this->ame_activation_email ] = '';
								update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
								break;
							case '103':
								$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								add_settings_error( 'api_key_exceeded_text', 'api_key_exceeded_error', 'Action required: for security reasons, Venus Assistant activation’s are issued 50 sites/domains at a time. Don’t worry, your activation’s are unlimited. Please <a target="_blank" href="https://seku.re/support">submit a quick support request</a> with the message "increase venus assistant activation limit". We will increase it for you immediately. Thank you and sorry for the delay.', 'error' );
								$options[ $this->ame_api_key ]          = '';
								$options[ $this->ame_activation_email ] = '';
								update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
								break;
							case '104':
								//$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								//add_settings_error( 'api_key_not_activated_text', 'api_key_not_activated_error', "{$activate_results['error']}. {$additional_info}", 'error' );
								//$options[ $this->ame_api_key ]          = '';
								//$options[ $this->ame_activation_email ] = '';
								update_option( $this->ame_activated_key, 'Activated' );
								update_option( $this->ame_deactivate_checkbox, 'off' );
								break;
							case '105':
								$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								add_settings_error( 'api_key_invalid_text', 'api_key_invalid_error', "{$activate_results['error']}. {$additional_info}", 'error' );
								$options[ $this->ame_api_key ]          = '';
								$options[ $this->ame_activation_email ] = '';
								update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
								break;
							case '106':
								$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
								add_settings_error( 'sub_not_active_text', 'sub_not_active_error', "{$activate_results['error']}. {$additional_info}", 'error' );
								$options[ $this->ame_api_key ]          = '';
								$options[ $this->ame_activation_email ] = '';
								update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
								break;
						}
					}
				} // End Plugin Activation
			}

			return $options;
		}

		/**
		 * Returns the API Key status from the WooCommerce API Manager on the server.
		 *
		 * @return array|mixed|object
		 */
		public function license_key_status() {
			$args = array(
				'email'       => $this->ame_options[ $this->ame_activation_email ],
				'licence_key' => $this->ame_options[ $this->ame_api_key ],
			);

			return json_decode( $this->status( $args ), true );
		}

		/**
		 * Deactivate the current API Key before activating the new API Key
		 *
		 * @param string $current_api_key
		 *
		 * @return bool
		 */
		public function replace_license_key( $current_api_key ) {
			$args = array(
				'email'       => $this->ame_options[ $this->ame_activation_email ],
				'licence_key' => $current_api_key,
			);

			$reset = $this->deactivate( $args ); // reset API Key activation

			if ( $reset == true ) {
				return true;
			}

			add_settings_error( 'not_deactivated_text', 'not_deactivated_error', __( 'The API Key could not be deactivated. Use the API Key Deactivation to manually deactivate the API Key before activating a new API Key. If all else fails, go to Plugins, then deactivate and reactivate this plugin, or if a theme change themes, then change back to this theme, then go to the Settings for this plugin/theme and enter the API Key information again to activate it. Also check the My Account dashboard to see if the API Key for this site was still active before the error message was displayed.', $this->text_domain ), 'updated' );

			return false;
		}

		// Deactivates the API Key to allow key to be used on another blog
		public function wc_am_license_key_deactivation( $input ) {
			$activation_status = get_option( $this->ame_activated_key );
			$args              = array(
				'email'       => $this->ame_options[ $this->ame_activation_email ],
				'licence_key' => $this->ame_options[ $this->ame_api_key ],
			);

			// For testing activation status_extra data
			// $activate_results = json_decode( $this->status( $args ), true );
			// print_r($activate_results); exit;

			$options = ( $input == 'on' ? 'on' : 'off' );
			if ( $options == 'on' && $activation_status == 'Activated' && $this->ame_options[ $this->ame_api_key ] != '' && $this->ame_options[ $this->ame_activation_email ] != '' ) {
				// deactivates API Key key activation
				$activate_results = json_decode( $this->deactivate( $args ), true );

				// Used to display results for development
				//print_r($activate_results); exit();

				if ( $activate_results[ 'deactivated' ] === true ) {
					$update        = array(
						$this->ame_api_key          => '',
						$this->ame_activation_email => ''
					);
					$merge_options = array_merge( $this->ame_options, $update );

					if ( ! empty( $this->ame_activated_key ) ) {
						update_option( $this->ame_data_key, $merge_options );
						update_option( $this->ame_activated_key, 'Deactivated' );
						add_settings_error( 'wc_am_deactivate_text', 'deactivate_msg', __( 'API Key deactivated. ', $this->text_domain ), 'update');
					}

					return $options;
				}

				if ( isset( $activate_results[ 'code' ] ) && ! empty( $this->ame_options ) && ! empty( $this->ame_activated_key ) ) {
					switch ( $activate_results[ 'code' ] ) {
						case '100':
							$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							add_settings_error( 'api_email_text', 'api_email_error', "{$activate_results['error']}. {$additional_info}", 'error' );
							$options[ $this->ame_activation_email ] = '';
							$options[ $this->ame_api_key ]          = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
							break;
						case '101':
							$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							add_settings_error( 'api_key_text', 'api_key_error', "{$activate_results['error']}. {$additional_info}", 'error' );
							$options[ $this->ame_api_key ]          = '';
							$options[ $this->ame_activation_email ] = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
							break;
						case '102':
							$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							add_settings_error( 'api_key_purchase_incomplete_text', 'api_key_purchase_incomplete_error', "{$activate_results['error']}. {$additional_info}", 'error' );
							$options[ $this->ame_api_key ]          = '';
							$options[ $this->ame_activation_email ] = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
							break;
						case '103':
							$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							add_settings_error( 'api_key_exceeded_text', 'api_key_exceeded_error', 'Action required: for security reasons, Venus Assistant activation’s are issued 50 sites/domains at a time. Don’t worry, your activation’s are unlimited. Please <a target="_blank" href="https://seku.re/support">submit a quick support request</a> with the message "increase venus assistant activation limit". We will increase it for you immediately. Thank you and sorry for the delay.', 'error' );
							$options[ $this->ame_api_key ]          = '';
							$options[ $this->ame_activation_email ] = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
							break;
						case '104':
							//$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							//add_settings_error( 'api_key_not_activated_text', 'api_key_not_activated_error', "{$activate_results['error']}. {$additional_info}", 'error' );
							// $options[ $this->ame_api_key ]          = '';
							// $options[ $this->ame_activation_email ] = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Activated' );
							break;
						case '105':
							$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							add_settings_error( 'api_key_invalid_text', 'api_key_invalid_error', "{$activate_results['error']}. {$additional_info}", 'error' );
							$options[ $this->ame_api_key ]          = '';
							$options[ $this->ame_activation_email ] = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
							break;
						case '106':
							$additional_info = ! empty( $activate_results[ 'additional info' ] ) ? esc_attr( $activate_results[ 'additional info' ] ) : '';
							add_settings_error( 'sub_not_active_text', 'sub_not_active_error', "{$activate_results['error']}. {$additional_info}", 'error' );
							$options[ $this->ame_api_key ]          = '';
							$options[ $this->ame_activation_email ] = '';
							update_option( $this->ame_options[ $this->ame_activated_key ], 'Deactivated' );
							break;
					}
				}
			} else {

				return $options;
			}

			return false;
		}

		public function wc_am_deactivate_text() { }

		public function wc_am_deactivate_textarea() {
			echo '<input type="checkbox" id="' . $this->ame_deactivate_checkbox . '" name="' . $this->ame_deactivate_checkbox . '" value="on"';
			echo checked( get_option( $this->ame_deactivate_checkbox ), 'on' );
			echo '/>';
			?><span class="description"><?php _e( 'Release this API Key.', $this->text_domain ); ?></span>
			<?php
		}

		/**
		 * Builds the URL containing the API query string for activation, deactivation, and status requests.
		 *
		 * @param array $args
		 *
		 * @return string
		 */
		public function create_software_api_url( $args ) {
			return add_query_arg( 'wc-api', 'am-software-api', $this->api_url ) . '&' . http_build_query( $args );
		}

		/**
		 * Sends the request to activate to the API Manager.
		 *
		 * @param array $args
		 *
		 * @return bool|string
		 */
		public function activate( $args ) {
			$defaults = array(
				'request'          => 'activation',
				'product_id'       => $this->ame_product_id,
				'instance'         => $this->ame_instance_id,
				'platform'         => $this->ame_domain,
				'software_version' => $this->ame_software_version
			);

			$args       = wp_parse_args( $defaults, $args );
			$target_url = esc_url_raw( $this->create_software_api_url( $args ) );
			$request    = wp_safe_remote_get( $target_url );

			// $request = wp_remote_post( $this->api_url . 'wc-api/am-software-api/', array( 'body' => $args ) );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 ) {
				// Request failed
				return false;
			}

			$response = wp_remote_retrieve_body( $request );

			return $response;
		}

		/**
		 * Sends the request to deactivate to the API Manager.
		 *
		 * @param array $args
		 *
		 * @return bool|string
		 */
		public function deactivate( $args ) {
			$defaults = array(
				'request'    => 'deactivation',
				'product_id' => $this->ame_product_id,
				'instance'   => $this->ame_instance_id,
				'platform'   => $this->ame_domain
			);

			$args       = wp_parse_args( $defaults, $args );
			$target_url = esc_url_raw( $this->create_software_api_url( $args ) );
			$request    = wp_safe_remote_get( $target_url );

			// $request = wp_remote_post( $this->api_url . 'wc-api/am-software-api/', array( 'body' => $args ) );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 ) {
				// Request failed
				return false;
			}

			$response = wp_remote_retrieve_body( $request );

			return $response;
		}

		/**
		 * Sends the status check request to the API Manager.
		 *
		 * @param array $args
		 *
		 * @return bool|string
		 */
		public function status( $args ) {
			$defaults = array(
				'request'    => 'status',
				'product_id' => $this->ame_product_id,
				'instance'   => $this->ame_instance_id,
				'platform'   => $this->ame_domain
			);

			$args       = wp_parse_args( $defaults, $args );
			$target_url = esc_url_raw( $this->create_software_api_url( $args ) );
			$request    = wp_safe_remote_get( $target_url );

			// $request = wp_remote_post( $this->api_url . 'wc-api/am-software-api/', array( 'body' => $args ) );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 ) {
				// Request failed
				return false;
			}

			$response = wp_remote_retrieve_body( $request );

			return $response;
		}

		/**
		 * Check for software updates.
		 */
		public function check_for_update() {
			$this->plugin_name = $this->ame_plugin_name;

			// Slug should be the same as the plugin/theme directory name
			if ( strpos( $this->plugin_name, '.php' ) !== 0 ) {
				$this->slug = dirname( $this->plugin_name );
			} else {
				$this->slug = $this->plugin_name;
			}

			/*********************************************************************
			 * The plugin and theme filters should not be active at the same time
			 *********************************************************************/
			/**
			 * More info:
			 * function set_site_transient moved from wp-includes/functions.php
			 * to wp-includes/option.php in WordPress 3.4
			 *
			 * set_site_transient() contains the pre_set_site_transient_{$transient} filter
			 * {$transient} is either update_plugins or update_themes
			 *
			 * Transient data for plugins and themes exist in the Options table:
			 * _site_transient_update_themes
			 * _site_transient_update_plugins
			 */

			// uses the flag above to determine if this is a plugin or a theme update request
			if ( $this->plugin_or_theme == 'plugin' ) {
				/**
				 * Plugin Updates
				 */
				// Check For Plugin Updates
				//add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'update_check' ) );
				// Check For Plugin Information to display on the update details page
				add_filter( 'plugins_api', array( $this, 'request' ), 10, 3 );
			} else if ( $this->plugin_or_theme == 'theme' ) {
				/**
				 * Theme Updates
				 */
				// Check For Theme Updates
				add_filter( 'pre_set_site_transient_update_themes', array( $this, 'update_check' ) );

				// Check For Theme Information to display on the update details page
				//add_filter( 'themes_api', array( $this, 'request' ), 10, 3 );

			}
		}

		/**
		 * Builds the URL containing the API query string for software update requests.
		 *
		 * @param array $args
		 *
		 * @return string
		 */
		private function create_upgrade_api_url( $args ) {
			return add_query_arg( 'wc-api', 'upgrade-api', $this->api_url ) . '&' . http_build_query( $args );
		}

		/**
		 * Check for updates against the remote server.
		 *
		 * @since  1.0.0
		 *
		 * @param  object $transient
		 *
		 * @return object $transient
		 */
		public function update_check( $transient ) {
			if ( empty( $transient->checked ) ) {
				return $transient;
			}

			$args = array(
				'request'          => 'pluginupdatecheck',
				'slug'             => $this->slug,
				'plugin_name'      => $this->plugin_name,
				//'version'			=>	$transient->checked[$this->plugin_name],
				'version'          => $this->ame_software_version,
				'product_id'       => $this->ame_product_id,
				'api_key'          => $this->ame_options[ $this->ame_api_key ],
				'activation_email' => $this->ame_options[ $this->ame_activation_email ],
				'instance'         => $this->ame_instance_id,
				'domain'           => $this->ame_domain,
				'software_version' => $this->ame_software_version,
				'extra'            => $this->extra,
			);

			// Check for a plugin update
			$response = $this->plugin_information( $args );
			// Displays an admin error message in the WordPress dashboard
			$this->check_response_for_errors( $response );

			// Set version variables
			if ( isset( $response ) && is_object( $response ) && $response !== false ) {
				// New plugin version from the API
				$new_ver = (string) $response->new_version;
				// Current installed plugin version
				$curr_ver = (string) $this->ame_software_version;
				//$curr_ver = (string)$transient->checked[$this->plugin_name];
			}

			// If there is a new version, modify the transient to reflect an update is available
			if ( isset( $new_ver ) && isset( $curr_ver ) ) {
				if ( $response !== false && version_compare( $new_ver, $curr_ver, '>' ) ) {
					if ( $this->plugin_or_theme == 'plugin' ) {
						$transient->response[ $this->plugin_name ] = $response;
					} else if ( $this->plugin_or_theme == 'theme' ) {
						$transient->response[ $this->plugin_name ][ 'new_version' ] = $response->new_version;
						$transient->response[ $this->plugin_name ][ 'url' ]         = $response->url;
						$transient->response[ $this->plugin_name ][ 'package' ]     = $response->package;
					}
				}
			}

			return $transient;
		}

		/**
		 * Sends and receives data to and from the server API
		 *
		 * @since  1.0.0
		 *
		 * @param array $args
		 *
		 * @return object $response
		 */
		public function plugin_information( $args ) {
			$target_url = esc_url_raw( $this->create_upgrade_api_url( $args ) );
			$request    = wp_safe_remote_get( $target_url );

			//$request = wp_remote_post( $this->api_url . 'wc-api/upgrade-api/', array( 'body' => $args ) );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 ) {
				return false;
			}

			$response = unserialize( wp_remote_retrieve_body( $request ) );

			/**
			 * For debugging errors from the API
			 * For errors like: unserialize(): Error at offset 0 of 170 bytes
			 * Comment out $response above first
			 */
			// $response = wp_remote_retrieve_body( $request );
			// print_r($response); exit;

			if ( is_object( $response ) ) {
				return $response;
			} else {
				return false;
			}
		}

		/**
		 * API request for informatin.
		 *
		 * If `$action` is 'query_plugins' or 'plugin_information', an object MUST be passed.
		 * If `$action` is 'hot_tags` or 'hot_categories', an array should be passed.
		 *
		 * @param false|object|array $result The result object or array. Default false.
		 * @param string             $action The type of information being requested from the Plugin Install API.
		 * @param object             $args
		 *
		 * @return object
		 */
		public function request( $result, $action, $args ) {
			// Is this a plugin or a theme?
			//if ( $this->plugin_or_theme == 'plugin' ) {
			//	$version = get_site_transient( 'update_plugins' );
			//} else if ( $this->plugin_or_theme == 'theme' ) {
			//
			//	$version = get_site_transient( 'update_themes' );
			//}

			// Check if this plugins API is about this plugin
			if ( isset( $args->slug ) ) {
				if ( $args->slug != $this->slug ) {
					return $result;
				}
			} else {
				return $result;
			}

			$args = array(
				'request'          => 'plugininformation',
				'plugin_name'      => $this->plugin_name,
				//'version'			=>	$version->checked[$this->plugin_name],
				'version'          => $this->ame_software_version,
				'product_id'       => $this->ame_product_id,
				'api_key'          => $this->ame_options[ $this->ame_api_key ],
				'activation_email' => $this->ame_options[ $this->ame_activation_email ],
				'instance'         => $this->ame_instance_id,
				'domain'           => $this->ame_domain,
				'software_version' => $this->ame_software_version,
				'extra'            => $this->extra,
			);

			$response = $this->plugin_information( $args );

			// If everything is okay return the $response
			if ( isset( $response ) && is_object( $response ) && $response !== false ) {
				return $response;
			}

			return $result;
		}

		/**
		 * Displays an admin error message in the WordPress dashboard
		 *
		 * @param  object $response
		 *
		 * @return string
		 */
		public function check_response_for_errors( $response ) {
			if ( ! empty( $response ) && is_object( $response ) ) {
				if ( isset( $response->errors[ 'no_key' ] ) && $response->errors[ 'no_key' ] == 'no_key' && isset( $response->errors[ 'no_subscription' ] ) && $response->errors[ 'no_subscription' ] == 'no_subscription' ) {
					add_action( 'admin_notices', array( $this, 'no_key_error_notice' ) );
					add_action( 'admin_notices', array( $this, 'no_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'exp_license' ] ) && $response->errors[ 'exp_license' ] == 'exp_license' ) {
					add_action( 'admin_notices', array( $this, 'expired_license_error_notice' ) );
				} else if ( isset( $response->errors[ 'hold_subscription' ] ) && $response->errors[ 'hold_subscription' ] == 'hold_subscription' ) {
					add_action( 'admin_notices', array( $this, 'on_hold_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'cancelled_subscription' ] ) && $response->errors[ 'cancelled_subscription' ] == 'cancelled_subscription' ) {
					add_action( 'admin_notices', array( $this, 'cancelled_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'exp_subscription' ] ) && $response->errors[ 'exp_subscription' ] == 'exp_subscription' ) {
					add_action( 'admin_notices', array( $this, 'expired_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'suspended_subscription' ] ) && $response->errors[ 'suspended_subscription' ] == 'suspended_subscription' ) {
					add_action( 'admin_notices', array( $this, 'suspended_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'pending_subscription' ] ) && $response->errors[ 'pending_subscription' ] == 'pending_subscription' ) {
					add_action( 'admin_notices', array( $this, 'pending_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'trash_subscription' ] ) && $response->errors[ 'trash_subscription' ] == 'trash_subscription' ) {
					add_action( 'admin_notices', array( $this, 'trash_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'no_subscription' ] ) && $response->errors[ 'no_subscription' ] == 'no_subscription' ) {
					add_action( 'admin_notices', array( $this, 'no_subscription_error_notice' ) );
				} else if ( isset( $response->errors[ 'no_activation' ] ) && $response->errors[ 'no_activation' ] == 'no_activation' ) {
					add_action( 'admin_notices', array( $this, 'no_activation_error_notice' ) );
				} else if ( isset( $response->errors[ 'no_key' ] ) && $response->errors[ 'no_key' ] == 'no_key' ) {
					add_action( 'admin_notices', array( $this, 'no_key_error_notice' ) );
				} else if ( isset( $response->errors[ 'download_revoked' ] ) && $response->errors[ 'download_revoked' ] == 'download_revoked' ) {
					add_action( 'admin_notices', array( $this, 'download_revoked_error_notice' ) );
				} else if ( isset( $response->errors[ 'switched_subscription' ] ) && $response->errors[ 'switched_subscription' ] == 'switched_subscription' ) {
					add_action( 'admin_notices', array( $this, 'switched_subscription_error_notice' ) );
				}
			}
		}

		/**
		 * Display license expired error notice
		 */
		public function expired_license_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The API key for %s has expired. You can reactivate or purchase an API key from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription on-hold error notice
		 */
		public function on_hold_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The subscription for %s is on-hold. You can reactivate the subscription from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription cancelled error notice
		 */
		public function cancelled_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The subscription for %s has been cancelled. You can renew the subscription from your account <a href="%s" target="_blank">dashboard</a>. A new API key will be emailed to you after your order has been completed.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription expired error notice
		 */
		public function expired_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The subscription for %s has expired. You can reactivate the subscription from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription expired error notice
		 */
		public function suspended_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The subscription for %s has been suspended. You can reactivate the subscription from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription expired error notice
		 */
		public function pending_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The subscription for %s is still pending. You can check on the status of the subscription from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription expired error notice
		 */
		public function trash_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'The subscription for %s has been placed in the trash and will be deleted soon. You can purchase a new subscription from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display subscription expired error notice
		 */
		public function no_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'A subscription for %s could not be found. You can purchase a subscription from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display missing key error notice
		 */
		public function no_key_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'A API key for %s could not be found. Maybe you forgot to enter an API key when setting up %s, or the key was deactivated in your account. You can reactivate or purchase an API key from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display missing download permission revoked error notice
		 */
		public function download_revoked_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'Download permission for %s has been revoked possibly due to an API key or subscription expiring. You can reactivate or purchase an API key from your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

		/**
		 * Display no activation error notice
		 */
		public function no_activation_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( '%s has not been activated. Go to the settings page and enter the API key and API email to activate %s.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_attr( $this->ame_software_product_id ) );
		}

		/**
		 * Display switched activation error notice
		 */
		public function switched_subscription_error_notice() {
			echo sprintf( '<div class="notice notice-info"><p>' . __( 'You changed the subscription for %s, so you will need to enter your new API Key in the settings page. The API Key should have arrived in your email inbox, if not you can get it by logging into your account <a href="%s" target="_blank">dashboard</a>.', $this->text_domain ) . '</p></div>', esc_attr( $this->ame_software_product_id ), esc_url( $this->ame_renew_license_url ) );
		}

	}
}